<?php
/**
 * ReDJ component for Joomla! 1.5
 *
 * @author Luigi Balzano (info@sistemistica.it)
 * @package ReDJ
 * @copyright Copyright 2009 - 2010
 * @license GNU Public License
 * @link http://www.sistemistica.it
 * @version 1.5.2 Community
 */

// Check to ensure this file is included in Joomla!  defined('_JEXEC') or die();
jimport('joomla.application.component.model');

class ReDJModelErrors extends JModel
{
  /**
   * Category data array
   *
   * @var array
   */
  var $_data = null;

  /**
   * Category total
   *
   * @var integer
   */
  var $_total = null;

  /**
   * Pagination object
   *
   * @var object
   */
  var $_pagination = null;

  /**
   * Constructor
   *
   * @since 1.5
   */
  function __construct()
  {
    parent::__construct();

    global $mainframe, $option;

    // Get the pagination request variables
    $limit = $mainframe->getUserStateFromRequest( $option.'e_limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
    $limitstart = $mainframe->getUserStateFromRequest( $option.'e_limitstart', 'limitstart', 0, 'int' );

    $this->setState('limit', $limit);
    $this->setState('limitstart', $limitstart);
  }

  /**
   * Method to get redj error item data
   *
   * @access public
   * @return array
   */
  function getData()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_data))
    {
      $query = $this->_buildQuery();
      $this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
    }

    return $this->_data;
  }

  /**
   * Method to get the total number of redj error items
   *
   * @access public
   * @return integer
   */
  function getTotal()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_total))
    {
      $query = $this->_buildQuery();
      $this->_total = $this->_getListCount($query);
    }

    return $this->_total;
  }

  /**
   * Method to get a pagination object for the redj errors
   *
   * @access public
   * @return integer
   */
  function getPagination()
  {
    // Lets load the content if it doesn't already exist
    if (empty($this->_pagination))
    {
      jimport('joomla.html.pagination');
      $this->_pagination = new JPagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
    }

    return $this->_pagination;
  }

  function _buildQuery()
  {
    // Get the WHERE and ORDER BY clauses for the query
    $where = $this->_buildContentWhere();
    $orderby = $this->_buildContentOrderBy();

    $query = ' SELECT e.id, e.visited_url, e.error_code, e.hits, e.last_visit, e.last_referer, "0" AS checked_out '
      . ' FROM #__redj_errors AS e '
      . $where
      . $orderby
    ;

    return $query;
  }

  function _buildContentOrderBy()
  {
    global $mainframe, $option;

    $filter_order = $mainframe->getUserStateFromRequest( $option.'e_filter_order', 'filter_order', '', 'cmd' );
    $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'e_filter_order_Dir', 'filter_order_Dir', '', 'word' );

    if ($filter_order != ''){
      $orderby = ' ORDER BY '.$filter_order.' '.$filter_order_Dir.', e.id';
    } else {
      $orderby = ' ORDER BY e.id';
    }

    return $orderby;
  }

  function _buildContentWhere()
  {
    global $mainframe, $option;

    $filter_state = $mainframe->getUserStateFromRequest( $option.'e_filter_state', 'e_filter_state', '', 'word' );
    $search = $mainframe->getUserStateFromRequest( $option.'e_search', 'e_search', '', 'string' );
    $search = JString::strtolower( $search );

    $where = array();

    if ($search) {
      $where[] = 'LOWER(e.visited_url) LIKE '.$this->_db->Quote('%'.$search.'%');
    }

    $where  = ( count( $where ) ? ' WHERE '. implode( ' AND ', $where ) : '' );

    return $where;
  }
}
?>
