<?php
/**
 * ReDJ plugin for Joomla! 1.5
 *
 * @author Luigi Balzano (info@sistemistica.it)
 * @package ReDJ
 * @copyright Copyright 2009 - 2010
 * @license GNU Public License
 * @link http://www.sistemistica.it
 * @version 1.5.2 Community
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

function strleft($s1, $s2) {
  return substr($s1, 0, strpos($s1, $s2));
}

function getPrefix() {
  $s = empty($_SERVER['HTTPS']) ? '' : ($_SERVER['HTTPS'] == "on") ? 's' : '';
  $protocol = strleft(strtolower($_SERVER['SERVER_PROTOCOL']), "/") . $s;
  $port = ($_SERVER['SERVER_PORT'] == '80') ? '' : (':' . $_SERVER['SERVER_PORT']);
  return $protocol . '://' . $_SERVER['SERVER_NAME'] . $port;
}

function selfURL() {
  return getPrefix() . $_SERVER['REQUEST_URI'];
}

function getHost($address) {
  $parseUrl = parse_url(trim($address));
  return @trim($parseUrl['host'] ? $parseUrl['host'] : array_shift(explode('/', $parseUrl['path'], 2)));
}

function getBase($pathonly = false) {
  /*
   * This is the same as JURI::base(), but the latter can't be absolutely called because
   * internally this calls JURI::getInstance, and this sets a static $instances['SERVER']
   * with server variables not modified yet. After this, these variables can't be overriden
   * anymore into the static array, so ReDJ isn't able to perform the "Internal Redirect"
   */
  if (strpos(php_sapi_name(), 'cgi') !== false && !empty($_SERVER['REQUEST_URI'])) {
    //Apache CGI
    $base_path =  rtrim(dirname(str_replace(array('"', '<', '>', "'"), '', $_SERVER["PHP_SELF"])), '/\\');
  } else {
    //Others
    $base_path =  rtrim(dirname($_SERVER['SCRIPT_NAME']), '/\\');
  }

  return $pathonly === false ? getPrefix().$base_path.'/' : $base_path;
}

function getMacroParamValue($macro, $param) {
  static $vars = array();
  static $paths = array();

  $value = '';
  switch ($macro) {
    case 'setvars':
      $vars = $param;
      break;
    case 'setpaths':
      $paths = $param;
      break;
    case 'querybuild':
    case 'querybuildfull':
      $build_vars = explode(',', $param);
      foreach ($build_vars as $k => $v) {
        $p = strpos($v, "=");
        if ($p === false) {
          // Only parameter name
          if (isset($vars[$v])) { // Need to check only not-null values
            $value .= (($value === '') ? '' : '&') . $v . '=' . $vars[$v];
          }
        } else {
          // New parameter or overrides existing
          $pn = substr($v, 0, $p);
          $pv = substr($v, $p + 1, strlen($v) - $p - 1);
          if ((strlen($pn) > 0) && (strlen($pv) > 0)) { // Need to take only not-null names and values
            $value .= (($value === '') ? '' : '&') . $pn . '=' . $pv;
          }
        }
      }
      if ( ($macro === 'querybuildfull') && ($value !== '') ) { $value = '?' . $value; }
      break;
    case 'queryvar':
      @$value = $vars[$param];
      break;
    case 'pathltrim':
      $value = $paths['path'];
      if (strpos($value, $param) === 0) {
        $value = substr($value, strlen($param), strlen($value) - strlen($param));
      }
      break;
    case 'pathrtrim':
      $value = $paths['path'];
      if (strpos($value, $param) === (strlen($value) - strlen($param))) {
        $value = substr($value, 0, strlen($value) - strlen($param));
      }
      break;
  }
  return $value;
}

class plgSystemReDJ extends JPlugin {
  /**
   * Constructor
   *
   * For php4 compatability we must not use the __constructor as a constructor for plugins
   * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
   * This causes problems with cross-referencing necessary for the observer design pattern.
   *
   * @param object $subject The object to observe
   * @since 1.5
   */
  function plgSystemReDJ(& $subject, $config) {
    parent :: __construct($subject, $config);
    JError::setErrorHandling(E_ERROR, 'callback', array('plgSystemReDJ', 'custom_error'));
  }

  function custom_error(& $object) {
    global $mainframe;

    if (JError::isError($object) == true) {
      $plugin = & JPluginHelper :: getPlugin('system', 'redj');
      $params = new JParameter($plugin->params);
      $customerror404page = $params->get('customerror404page', 0);
      $error404pageid = $params->get('error404pageid', "");
      $trackerrors = $params->get('trackerrors', 0);
      $redirectallerrors = $params->get('redirectallerrors', 0);
      $redirectallerrorsurl = $params->get('redirectallerrorsurl', "");
      $db = JFactory::getDBO();

      if ($trackerrors) {
        // Track error URL calls
        $visited_url = selfURL();
        $error_code = $object->code;
        $last_visit = date("Y-m-d H:i:s");
        $last_referer = @$_SERVER['HTTP_REFERER'];
        $db->setQuery("INSERT INTO #__redj_errors (`visited_url`, `error_code`, `hits`, `last_visit`, `last_referer`) VALUES (" . $db->quote( $visited_url ) . ", " . $db->quote( $error_code ) . ", 1, " . $db->quote( $last_visit ) . ", " . $db->quote( $last_referer ) . ") ON DUPLICATE KEY UPDATE `hits` = `hits` + 1, `last_visit` = " . $db->quote( $last_visit ) . ", `last_referer` = " . $db->quote( $last_referer ));
        $res = @$db->query();
      }

      if ( ($redirectallerrors) && (!empty($redirectallerrorsurl)) ){
        $siteurl = str_replace( 'https://', '', getBase() );
        $siteurl = rtrim ( str_replace( 'http://', '', $siteurl ), '/');
        $tourl = str_replace( '{siteurl}', $siteurl, $redirectallerrorsurl );
        @ob_end_clean();
        header("HTTP/1.1 301 Moved Permanently");
        header('location: ' . $tourl);
        exit;
      }

      $db->setQuery("SELECT * FROM #__redj_pages WHERE id=" . $db->quote( $error404pageid ));
      $items = $db->loadObjectList();

      if (($object->code == 404) && ($customerror404page) && (count($items) > 0) && (!empty($items[0]->page))) { // Custom error handler
        // Update hits
        $last_visit = date("Y-m-d H:i:s");
        $db->setQuery("UPDATE #__redj_pages SET hits = hits + 1, last_visit = " . $db->quote( $last_visit ) . " WHERE id = " . $db->quote( $error404pageid ));
        $res = @$db->query();
        // Initialize variables
        jimport('joomla.document.document');
        $app       = & JFactory::getApplication();
        $document  = & JDocument::getInstance('error');
        $config    = & JFactory::getConfig();

        // Get the current template from the application
        $template = $app->getTemplate();

        // Push the error object into the document
        $document->setError($object);

        @ob_end_clean();
        // Call default render to set header
        $data = $document->render(false, array (
          'template' => $template,
          'directory' => JPATH_THEMES,
          'debug' => $config->getValue('config.debug')
        ));
        // Replace macros in custom body
        $custombody = $items[0]->page;
        $siteurl = str_replace( 'https://', '', getBase() );
        $siteurl = rtrim ( str_replace( 'http://', '', $siteurl ), '/');
        $custombody = str_replace( '{siteurl}', $siteurl, $custombody );
        $sitename = $mainframe->getCfg('sitename');
        $custombody = str_replace( '{sitename}', $sitename, $custombody );
        $sitemail = $mainframe->getCfg('mailfrom');
        $custombody = str_replace( '{sitemail}', $sitemail, $custombody );
        $errormessage = $object->message;
        $custombody = str_replace( '{errormessage}', $errormessage, $custombody );
        // Also replace {article} macro
        $regex_base = '\{(article)\s+(\d+)\}';
        $regex_search = "/(.*?)$regex_base(.*)/si";
        $regex_replace = "/$regex_base/si";
        $found = preg_match( $regex_search, $custombody, $matches );
        while ($found) {
          $content_id = $matches[3];
          $content_query = "SELECT `introtext`, `fulltext` FROM #__content WHERE `id`=".(int)$content_id;
          $content_db = & JFactory::getDBO();
          $content_db->setQuery( $content_query );
          $content_row = $content_db->loadObject();
          if (is_object($content_row)) {
            $content = $content_row->introtext . $content_row->fulltext;
          } else {
            $content = '';
          }
          $custombody = preg_replace( $regex_replace, $content, $custombody ); // Replace all occurrences
          $found = preg_match( $regex_search, $custombody, $matches );
        }

        // Set the custom body
        JResponse::setBody($custombody);
        echo JResponse::toString();
        $app->close(0);
      } else {
        // Restore default error handler
        JError::setErrorHandling(E_ERROR, 'callback', array('JError', 'customErrorPage'));
        // Re-raise original error...
        JError::raise($object->level, $object->code, $object->message, $object->info, $object->backtrace);
      }
    }
  }

  function onAfterInitialise() {
    global $mainframe;

    // Only mod site pages that are html docs (no admin, install, etc.)
    if ( !$mainframe->isSite() ) return;
    if ( JRequest::getBool('no_html') === true ) return;

    $plugin = & JPluginHelper :: getPlugin('system', 'redj');
    $params = new JParameter($plugin->params);
    $trackreferers = $params->get('trackreferers', 0);
    $db = JFactory::getDBO();
    $visited_url = selfURL(); // Here because needed for both 'track referers' and 'replace macros'

    if ($trackreferers) {
      // Track referers URL calls
      $excludereferers = str_replace( '{self}', $_SERVER['SERVER_NAME'], $params->get('excludereferers', '') );
      $excludereferers = str_replace( '{none}', '', $excludereferers );
      $excludereferers = explode("\n", $excludereferers);
      $referer_url = @$_SERVER['HTTP_REFERER'];
      $domain = getHost($referer_url);
      if ( !in_array($domain, $excludereferers) ) {
        $last_visit = date("Y-m-d H:i:s");

        $db->setQuery("INSERT IGNORE INTO `#__redj_visited_urls` (`id`, `visited_url`) VALUES (NULL, " . $db->quote( $visited_url ) . ")");
        $res = @$db->query();

        $db->setQuery("INSERT IGNORE INTO `#__redj_referer_urls` (`id`, `referer_url`, `domain`) VALUES (NULL, " . $db->quote( $referer_url ) . ", " . $db->quote( $domain ) . ")");
        $res = @$db->query();

        $db->setQuery("INSERT INTO `#__redj_referers` (`id`, `visited_url`, `referer_url`, `hits`, `last_visit`) VALUES (NULL, (SELECT `id` FROM `#__redj_visited_urls` WHERE `visited_url` = " . $db->quote( $visited_url ) . "), (SELECT `id` FROM `#__redj_referer_urls` WHERE `referer_url` = " . $db->quote( $referer_url ) . "), 1, " . $db->quote( $last_visit ) . " ) ON DUPLICATE KEY UPDATE `hits` = `hits` + 1, `last_visit` = " . $db->quote( $last_visit ));
        $res = @$db->query();
      }
    }

    $currenturi = $_SERVER['REQUEST_URI'];
    $db->setQuery("SELECT * FROM #__redj WHERE ( ((" . $db->quote( $currenturi ) . " REGEXP BINARY fromurl)>0 and case_sensitive<>'0') or ((" . $db->quote( $currenturi ) . " REGEXP fromurl)>0 and case_sensitive='0') ) and published='1' ORDER BY ordering");
    $items = $db->loadObjectList();
    if ( count($items) > 0 ) {
      // Notes: if more than one match with current URI, we takes only the first one with ordering set
      if ( !empty($items[0]->tourl) ) {
        // Update hits
        $last_visit = date("Y-m-d H:i:s");
        $db->setQuery("UPDATE #__redj SET hits = hits + 1, last_visit = " . $db->quote( $last_visit ) . " WHERE id = " . $db->quote( $items[0]->id ));
        $res = @$db->query();
        $siteurl = str_replace( 'https://', '', getBase() );
        $siteurl = rtrim ( str_replace( 'http://', '', $siteurl ), '/');

        // Initialize macro related variables
        $uri_visited_url =& JURI::getInstance($visited_url);
        getMacroParamValue('setvars', $uri_visited_url->_vars);
        $authority = (isset($uri_visited_url->_port)) ? $uri_visited_url->_host . ':' . $uri_visited_url->_port : $uri_visited_url->_host;
        $authority = (isset($uri_visited_url->_user)) ? $uri_visited_url->_user . ':' . $uri_visited_url->_pass . '@' . $authority : $authority;
        $baseonly = getBase(true);
        $pathfrombase = (strlen($baseonly) > 0) ? substr($uri_visited_url->_path, strlen($baseonly), strlen($uri_visited_url->_path) - strlen($baseonly)) : $uri_visited_url->_path;
        getMacroParamValue('setpaths', array('path' => $uri_visited_url->_path, 'pathfrombase' => $pathfrombase));

        /*
          http://fredbloggs:itsasecret@www.example.com:8080/path/to/Joomla/section/cat/index.php?task=view&id=32#anchorthis
          \__/   \________/ \________/ \_____________/ \__/\___________________________________/ \_____________/ \________/
           |          |         |              |        |                   |                           |             |
         scheme     user       pass          host      port                path                       query       fragment

        Supported macro for destination URL:
           {siteurl}                                                    www.example.com/path/to/Joomla
           {scheme}                                                     http
           {host}                                                       www.example.com
           {port}                                                       8080
           {user}                                                       fredbloggs
           {pass}                                                       itsasecret
           {path}                                                       /path/to/Joomla/section/cat/index.php
           {query}                                                      task=view&id=32
           {queryfull}                                                  ?task=view&id=32
           {querybuild id,task}                                         id=32&task=view
           {querybuild id,task=edit}                                    id=32&task=edit
           {querybuild id,task=view,ItemId=12}                          id=32&task=view&ItemId=12
           {querybuildfull id,task}                                     ?id=32&task=view
           {querybuildfull id,task=save}                                ?id=32&task=save
           {querybuildfull id,task,action=close}                        ?id=32&task=view&action=close
           {querydrop task}                                             id=32
           {querydrop id,task=edit}                                     task=edit
           {querydrop id,task=save,action=close}                        task=save&action=close
           {querydropfull task}                                         ?id=32
           {querydropfull id,task=save}                                 ?task=save
           {querydropfull id,task=edit,action=close}                    ?task=edit&action=close
           {queryvar task}                                              view
           {queryvar id}                                                32
           {authority}                                                  fredbloggs:itsasecret@www.example.com:8080
           {baseonly}                                                   /path/to/Joomla (empty when installed on root, i.e. it will never contains a trailing slash)
           {pathfrombase}                                               /section/cat/index.php
           {pathltrim /path/to}                                         /Joomla/section/cat/index.php
           {pathrtrim /index.php}                                       /path/to/Joomla/section/cat
           {pathfrombaseltrim /section}                                 /cat/index.php
           {pathfrombasertrim index.php}                                /section/cat/
           {preg_match N}pattern{/preg_match}                           (return the N-th matched pattern on the full source url, where N = 0 when not specified)
           {preg_match}/([^\/]+)(\.php|\.html|\.htm)/i{/preg_match}     index.php
           {preg_match 2}/([^\/]+)(\.php|\.html|\.htm)/i{/preg_match}   .php
        */

        // Supported macros patterns
        $patterns = array();
        $patterns[0] = "/\{siteurl\}/";
        $patterns[1] = "/\{scheme\}/";
        $patterns[2] = "/\{host\}/";
        $patterns[3] = "/\{port\}/";
        $patterns[4] = "/\{user\}/";
        $patterns[5] = "/\{pass\}/";
        $patterns[6] = "/\{path\}/";
        $patterns[7] = "/\{query\}/";
        $patterns[8] = "/\{queryfull\}/";
        $patterns[9] = "/\{querybuild ([^\}]+)\}/e";
        $patterns[10] = "/\{querybuildfull ([^\}]+)\}/e";
        $patterns[13] = "/\{queryvar ([^\}]+)\}/e";
        $patterns[14] = "/\{authority\}/";
        $patterns[15] = "/\{baseonly\}/";
        $patterns[16] = "/\{pathfrombase\}/";
        $patterns[17] = "/\{pathltrim ([^\}]+)\}/e";
        $patterns[18] = "/\{pathrtrim ([^\}]+)\}/e";

        // Supported macros replacements
        $replacements = array();
        $replacements[0] = $siteurl;
        $replacements[1] = $uri_visited_url->_scheme;
        $replacements[2] = $uri_visited_url->_host;
        $replacements[3] = $uri_visited_url->_port;
        $replacements[4] = $uri_visited_url->_user;
        $replacements[5] = $uri_visited_url->_pass;
        $replacements[6] = $uri_visited_url->_path;
        $replacements[7] = $uri_visited_url->_query;
        $replacements[8] = (isset($uri_visited_url->_query)) ? '?' . $uri_visited_url->_query : '';
        $replacements[9] = "getMacroParamValue('querybuild', '\\1')";
        $replacements[10] = "getMacroParamValue('querybuildfull', '\\1')";
        $replacements[13] = "getMacroParamValue('queryvar', '\\1')";
        $replacements[14] = $authority;
        $replacements[15] = $baseonly;
        $replacements[16] = $pathfrombase;
        $replacements[17] = "getMacroParamValue('pathltrim', '\\1')";
        $replacements[18] = "getMacroParamValue('pathrtrim', '\\1')";

        $tourl = preg_replace($patterns, $replacements, $items[0]->tourl);

        @ob_end_clean();
        switch ($items[0]->redirect) {
          case 307:
            header("HTTP/1.1 307 Temporary Redirect");
            header('location: ' . $tourl);
            exit;
          default:
            header("HTTP/1.1 301 Moved Permanently");
            header('location: ' . $tourl);
            exit;
        }
      }
    }

  }

}
