<?php
/**
 * Tag Meta plugin for Joomla! 1.5
 *
 * @author Luigi Balzano (info@sistemistica.it)
 * @package TagMeta
 * @copyright Copyright 2009 - 2010
 * @license GNU Public License
 * @link http://www.sistemistica.it
 * @version 1.3 Community
 */

/**
 *
 * Tag Meta allows to manipulate page meta info (e.g. tag 'title', meta tag 'description', and so on)
 * Also supports the 'canonical' page link and the following 'robots' tag:
 *
 * noindex
 *     Don't index the page
 * nofollow
 *     Don't follow links found in the page
 * nosnippet
 *     Don't show a short description for the page in search results
 * noarchive
 *     Don't save a copy cache of the page
 * noodp
 *     Don't use Dmoz description in search resultsm even if present
 *
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

class plgSystemTagMeta extends JPlugin {

/**
*
* @var boolean
* @access  private
*/
var $_clean_generator = false;

/**
*
* @var string
* @access  private
*/
var $_eol = "\12";

  /**
   * Constructor
   *
   * For php4 compatability we must not use the __constructor as a constructor for plugins
   * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
   * This causes problems with cross-referencing necessary for the observer design pattern.
   *
   * @param object $subject The object to observe
   * @since 1.5
   */
  function plgSystemTagMeta(& $subject, $config) {
    parent :: __construct($subject, $config);
  }

  function onAfterDispatch() {
    global $mainframe;

    $document = & JFactory::getDocument();
    $docType = $document->getType();
    $this->_eol = $document->_getLineEnd();
    // Only mod site pages that are html docs (no admin, install, etc.)
    if ( !$mainframe->isSite() ) return;
    if ( $docType != 'html' ) return;

    $plugin = & JPluginHelper :: getPlugin('system', 'tagmeta');
    $params = new JParameter($plugin->params);

    // Set macro values
    $siteurl = str_replace( 'https://', '', JURI::base() );
    $siteurl = rtrim ( str_replace( 'http://', '', $siteurl ), '/');

    // Supported macros patterns
    $patterns = array();
    $patterns[0] = "/\{siteurl\}/";
    $patterns[1] = "/\{sitename\}/";
    $patterns[2] = "/\{globaldescription\}/";
    $patterns[3] = "/\{globalkeywords\}/";
    $patterns[4] = "/\{currenttitle\}/";
    $patterns[5] = "/\{currentdescription\}/";
    $patterns[6] = "/\{currentkeywords\}/";
    $patterns[7] = "/\{currentauthor\}/";
    $patterns[8] = "/\{currentgenerator\}/";

    // Supported macros replacements
    $replacements = array();
    $replacements[0] = $siteurl;
    $replacements[1] = $mainframe->getCfg('sitename');
    $replacements[2] = $mainframe->getCfg('MetaDesc');
    $replacements[3] = $mainframe->getCfg('MetaKeys');
    $replacements[4] = $document->getTitle();
    $replacements[5] = $document->getDescription();
    $replacements[6] = $document->getMetaData('keywords');
    $replacements[7] = $document->getMetaData('author');
    $replacements[8] = $document->getGenerator();

    if ( ($params->get('redirect', 0) == 1) && ( (isset($_SERVER['REDIRECT_URL'])) || (isset($_SERVER['HTTP_X_REWRITE_URL'])) ) ) {
      $currenturi = (isset($_SERVER['HTTP_X_REWRITE_URL'])) ? $_SERVER['HTTP_X_REWRITE_URL'] : $_SERVER['REDIRECT_URL'];
    } else {
      $currenturi = $_SERVER['REQUEST_URI'];
    }

    // Determine if the request was over SSL (HTTPS)
    if (isset($_SERVER['HTTPS']) && !empty($_SERVER['HTTPS']) && (strtolower($_SERVER['HTTPS']) != 'off')) {
      $https = 's://';
    } else {
      $https = '://';
    }
    $theURI = 'http' . $https . $_SERVER['HTTP_HOST'] . $currenturi;

    // Now we need to clean what we got since we can't trust the server var
    $theURI = urldecode($theURI);
    $theURI = str_replace('"', '&quot;',$theURI);
    $theURI = str_replace('<', '&lt;',$theURI);
    $theURI = str_replace('>', '&gt;',$theURI);
    $theURI = preg_replace('/eval\((.*)\)/', '', $theURI);
    $theURI = preg_replace('/[\\\"\\\'][\\s]*javascript:(.*)[\\\"\\\']/', '""', $theURI);

    $db = JFactory::getDBO();
    $db->setQuery("SELECT * FROM #__tagmeta WHERE ( ((" . $db->quote( $currenturi ) . " REGEXP BINARY uri)>0 and case_sensitive<>'0') or ((" . $db->quote( $currenturi ) . " REGEXP uri)>0 and case_sensitive='0') ) and published='1' ORDER BY ordering");
    $items = $db->loadObjectList();
    if ( count($items) > 0 ) {
      // Notes: if more than one match with current URI, we takes only the first one with ordering set

      // Update hits
      $last_visit = date("Y-m-d H:i:s");
      $db->setQuery("UPDATE #__tagmeta SET hits = hits + 1, last_visit = " . $db->quote( $last_visit ) . " WHERE id = " . $db->quote( $items[0]->id ));
      $res = @$db->query();

      // Replace macros
      $items[0]->title = preg_replace($patterns, $replacements, $items[0]->title);
      $items[0]->description = preg_replace($patterns, $replacements, $items[0]->description);
      $items[0]->author = preg_replace($patterns, $replacements, $items[0]->author);
      $items[0]->keywords = preg_replace($patterns, $replacements, $items[0]->keywords);
      $items[0]->canonical = str_replace( '{siteurl}', $siteurl, $items[0]->canonical ); // Only {siteurl} macro is supported for 'canonical'

      if ( !empty($items[0]->title) ) { $document->setTitle($items[0]->title); }
      if ( !empty($items[0]->description) ) { $document->setDescription(str_replace('"', '&quot;', $items[0]->description)); }
      if ( !empty($items[0]->author) ) { $document->setMetaData('author', $items[0]->author); }
      if ( !empty($items[0]->keywords) ) { $document->setMetaData('keywords', str_replace('"', '&quot;', $items[0]->keywords)); }
      if ( !empty($items[0]->canonical) ) { $document->addHeadLink($items[0]->canonical, 'canonical', 'rel'); }

      // Robots meta options: 0=No,1=Yes,2=Skip
      $robots = '';
      if ($items[0]->rindex != 2) { $robots .= ($items[0]->rindex) ? 'index,' : 'noindex,'; }
      if ($items[0]->rfollow != 2) { $robots .= ($items[0]->rfollow) ? 'follow,' : 'nofollow,'; }
      if ($items[0]->rsnippet != 2) { $robots .= ($items[0]->rsnippet) ? 'snippet,' : 'nosnippet,'; }
      if ($items[0]->rarchive != 2) { $robots .= ($items[0]->rarchive) ? 'archive,' : 'noarchive,'; }
      if ($items[0]->rodp != 2) { $robots .= ($items[0]->rodp) ? 'odp' : 'noodp'; }
      $robots = rtrim($robots, ','); // Drop last char if is a comma
      if ( !empty($robots) ) { $document->setMetaData('robots', $robots); }
    }

    $replacegenerator = $params->get('replacegenerator', 0);
    if ($replacegenerator != 0) {
      if ($replacegenerator == 3) {
        if ($document->getMetaData('generator')) { $document->setGenerator(''); $this->_clean_generator = true; } // Clean if exists
      } else {
        $customgenerator = $params->get('customgenerator', '');
        if ( (($document->getMetaData('generator')) && ($replacegenerator == 1)) || ($replacegenerator == 2) ) {
          $document->setGenerator(str_replace('"', '&quot;', $customgenerator)); // Replace existing or force
        }
      }
    }

    $addsitename = $params->get('addsitename', 0);
    if ($addsitename != 0) {
      // Add site name before or after the page title
      $sitename = $mainframe->getCfg('sitename');
      $separator = str_replace( '\b', ' ', $this->params->get('separator') );
      $currenttitle = $document->getTitle();
      if ( $addsitename == 1 ) {
        $newtitle = $sitename . $separator . $currenttitle; // Before
      } else {
        $newtitle = $currenttitle . $separator . $sitename; // After
      }
      $document->setTitle( htmlspecialchars_decode($newtitle) );
    }

    if ( $this->params->get('cleandefaultpage') == 1 ) {
      // Check if this is the default page (home page)
      $menu =& JSite::getMenu();
      if ( $menu->getActive() == $menu->getDefault() ) {
        $sitename = $mainframe->getCfg('sitename');
        $document->setTitle($sitename);
      }
    }

    $metatitle = $params->get('metatitle', 1);
    if ( $metatitle ) {
          $tagvalue = $document->getTitle();
          if ( empty($tagvalue) ) {
                // Tag title is empty
                $metavalue = $document->getMetaData('title');
                if ( ( !empty($metavalue) ) && ($metatitle == 2) ) {
                  $document->setTitle($metavalue);
                }
          } else {
                // Tag title is not empty
                $metavalue = $document->getMetaData('title');
                if ( ( !empty($metavalue) ) || ($metatitle == 2) ) {
                  $document->setMetaData('title', str_replace('"', '&quot;', $tagvalue));
                }
          }
    }

    $customauthor = $params->get('customauthor', '');
    $addauthor = $params->get('addauthor', 0);
    if ( ( $customauthor ) && ($addauthor != 0) ) {
          $currentauthor = $document->getMetaData('author');
          if ( ($addauthor == 1) || ( empty($currentauthor) ) ) {
                $document->setMetaData('author', str_replace('"', '&quot;', $customauthor));
          }
    }

  }

  function onAfterRender(){
    global $mainframe;

    // Only mod site pages that are html docs (no admin, install, etc.)
    if ( !$mainframe->isSite() ) return;
    if ( JRequest::getBool('no_html') === true ) return;

    $content = JResponse::getBody();
    $changed = false;

    // Clean meta tag generator
    if ($this->_clean_generator) {
      $content = preg_replace('/<meta.*name=[\",\']generator[\",\'].*\/?>/i', '', $content);
      $changed = true;
    }

    if ($changed) { JResponse::setBody($content); }
  }

}
